package com.yanzuoguang.cloud.aop;

import com.alibaba.fastjson.JSON;
import com.yanzuoguang.util.cache.MemoryCache;
import com.yanzuoguang.util.contants.ResultConstants;
import com.yanzuoguang.util.helper.StringHelper;
import com.yanzuoguang.util.thread.ThreadNext;
import com.yanzuoguang.util.vo.LogVo;
import com.yanzuoguang.util.vo.ResponseResult;
import org.aspectj.lang.ProceedingJoinPoint;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * 基本处理拦截类
 * @author 颜佐光
 */
public class BaseRequestAspect implements ThreadNext.Next {


    @Value("${spring.application.name}")
    protected String applicationName;

    @Value("${yzg.logAll:false}")
    protected boolean logAll = false;

    @Value("${yzg.cacheTime:120}")
    protected int cacheTime = 120;

    @Autowired
    protected ApplicationContext context;

    protected LogFeign logFeign;
    /**
     * 缓存队列
     */
    protected volatile LinkedBlockingQueue<LogVo> cache = new LinkedBlockingQueue<LogVo>();

    /**
     * 缓存的每次请求的结果
     */
    protected MemoryCache<RequestCacheResult> cacheResult = new MemoryCache<>(cacheTime);

    public BaseRequestAspect() {
        ThreadNext.start(this, "save log error");
    }

    /**
     * 添加日志到缓存中，并不是立即添加，而是通过线程执行，防止对环境造成影响
     *
     * @param logVo
     */
    public void addLog(LogVo logVo) {
        cache.add(logVo);
    }

    /**
     * 初始化日志Vo
     *
     * @param url            请求路径
     * @param joinPoint      请求参数
     * @param responseResult 返回参数
     * @return
     */
    protected LogVo initLogInterVo(String url, ProceedingJoinPoint joinPoint, ResponseResult responseResult) {
        LogVo logInterVo = new LogVo();
        logInterVo.setLogId(StringHelper.getNewID());
        //平台名
        logInterVo.setLogSources(applicationName);
        //请求URL
        logInterVo.setInterUrl(url);
        List<Object> para = new ArrayList<>();
        for (Object item : joinPoint.getArgs()) {
            if (item instanceof HttpServlet || item instanceof HttpServletResponse || item instanceof HttpServletRequest) {
                continue;
            }
            para.add(item);
        }
        Object paraTo = para;
        if (para.size() == 1) {
            paraTo = para.get(0);
        }
        //请求参数
        logInterVo.setContent(JSON.toJSONString(paraTo));
        //返回参数
        logInterVo.setResult(JSON.toJSONString(responseResult));
        logInterVo.setStatus(responseResult != null && responseResult.getCode() == ResultConstants.SUCCESS ? 1 : 0);
        return logInterVo;
    }

    /**
     * 执行下一个函数,出现异常会继续执行
     *
     * @return 是否继续执行
     */
    @Override
    public boolean next() {
        while (cache.size() > 0) {
            LogVo item = cache.poll();
            if (item != null) {
                if (logFeign == null) {
                    logFeign = context.getBean(LogFeign.class);
                }
                logFeign.save(item);
            }
        }
        return true;
    }

    /**
     * 沉睡时间
     *
     * @return
     */
    @Override
    public int getNextTime() {
        return 1000;
    }
}
